#!/usr/bin/env bash

CUR_IFS=$IFS

script_name="$(basename $(realpath $0))"
script_dir="$(dirname $(realpath $0))"

conf_dir="${script_dir}/conf"
snippet_dir="${script_dir}/snippets"

declare -a unsorted_website_arr
declare -a website_arr

log_file="$(mktemp)"

backup_date=$(date +%Y-%m-%d-%H%M)


# =============
# --- Some functions
# =============

clean_up() {


   if [[ -f "$_backup_crontab_file" ]]; then

      echononl "(Re)Install previously saved crontab from '$_backup_crontab_file'.."

      crontab $_backup_crontab_file >> $log_file 2>&1

      if [[ $? -eq 0 ]]; then
         echo_ok
      else
         echo_failed
         error "$(cat $log_file)"
      fi

   fi

   # Perform program exit housekeeping
   rm -f $log_file
   blank_line
   exit $1
}

is_number() {

   return $(test ! -z "${1##*[!0-9]*}" > /dev/null 2>&1);

   # - also possible
   # -
   #[[ ! -z "${1##*[!0-9]*}" ]] && return 0 || return 1
   #return $([[ ! -z "${1##*[!0-9]*}" ]])
}

echononl(){
   echo X\\c > /tmp/shprompt$$
   if [ `wc -c /tmp/shprompt$$ | awk '{print $1}'` -eq 1 ]; then
      echo -e -n "$*\\c" 1>&2
   else
       echo -e -n "$*" 1>&2
   fi
   rm /tmp/shprompt$$
}
echo_done() {
   if $terminal ; then
      echo -e "\033[75G[ \033[32mdone\033[m ]"
   else
      echo " [ done ]"
   fi
}
echo_ok() {
   if $terminal ; then
      echo -e "\033[75G[ \033[32mok\033[m ]"
   else
      echo " [ ok ]"
   fi
}
echo_warning() {
   if $terminal ; then
      echo -e "\033[75G[ \033[33m\033[1mwarn\033[m ]"
   else
      echo " [ warning ]"
   fi
}
echo_failed(){
   if $terminal ; then
      echo -e "\033[75G[ \033[1;31mfailed\033[m ]"
   else
      echo ' [ failed! ]'
   fi
}
echo_skipped() {
   if $terminal ; then
      echo -e "\033[75G[ \033[37mskipped\033[m ]"
   else
      echo " [ skipped ]"
   fi
}
fatal (){
   echo ""
   echo ""
   if $terminal ; then
      echo -e "\t[ \033[31m\033[1mFatal\033[m ]: \033[37m\033[1m$*\033[m"
      echo ""
      echo -e "\t\033[31m\033[1m           Script will be interrupted..\033[m\033[m"
   else
      echo "fatal: $*"
      echo "Script will be interrupted.."
   fi
   clean_up 1
}

error(){
   echo ""
   if $terminal ; then
      echo -e "\t[ \033[31m\033[1mFehler\033[m ]: $*"
   else
      echo "Error: $*"
   fi
   echo ""
}

warn (){
   echo ""
   if $terminal ; then
      echo -e "\t[ \033[33m\033[1mWarning\033[m ]: $*"
   else
      echo "Warning: $*"
   fi
   echo ""
}

info (){
   echo ""
   if $terminal ; then
      echo -e "\t[ \033[32m\033[1mInfo\033[m ]: $*"
   else
      echo "Info: $*"
   fi
   echo ""
}

detect_os_1 () {

   if $(which lsb_release > /dev/null 2>&1) ; then

      os_dist="$(lsb_release -i | awk '{print tolower($3)}')"
      os_version="$(lsb_release -r | awk '{print tolower($2)}')"
      os_codename="$(lsb_release -c | awk '{print tolower($2)}')"

      if [[ "$os_dist" = "debian" ]]; then
         if $(echo "$os_version" | grep -q '\.') ; then
            os_version=$(echo "$os_version" | cut --delimiter='.' -f1)
         fi
      fi

   elif [[ -e "/etc/os-release" ]]; then

      . /etc/os-release

      os_dist=$ID
      os_version=${VERSION_ID}

   fi

   # remove whitespace from os_dist and os_version
   os_dist="${os_dist// /}"
   os_version="${os_version// /}"

}

# - Remove leading/trailling whitespaces
# -
trim() {
    local var="$*"
    var="${var#"${var%%[![:space:]]*}"}"   # remove leading whitespace characters
    var="${var%"${var##*[![:space:]]}"}"   # remove trailing whitespace characters
    echo -n "$var"
}

## - Check if a given array (parameter 2) contains a given string (parameter 1)
## -
containsElement () {
   local e
   for e in "${@:2}"; do [[ "$e" == "$1" ]] && return 0; done
   return 1
}

blank_line() {
   if $terminal ; then
      echo ""
   fi
}



# ----------
# - Jobhandling 
# ----------

# - Run 'clean_up' for signals SIGHUP SIGINT SIGTERM
# -
trap clean_up SIGHUP SIGINT SIGTERM

## - 
while IFS='' read -r -d '' _conf_file ; do
   source $_conf_file
   if [[ -n "$WEBSITE" ]] ; then
      unsorted_website_arr+=("${WEBSITE}:$_conf_file")
   fi
   WEBSITE=""
done  < <(find "${conf_dir}" -maxdepth 1 -type f -name "*.conf" -print0)

if [[ ${#unsorted_website_arr} -eq 0 ]]; then
   fatal "No configuration files found in '${script_dir}/conf' or no website configured!"
fi

# - Sort array
# -
IFS=$'\n' website_arr=($(sort <<<"${unsorted_website_arr[*]}"))


# - Reset IFS
# -
IFS=$CUR_IFS



# =============
# --- Some 
# =============

# - Support systemd ?
# -
if [[ "X$(which systemd)" = "X" ]]; then
   SYSTEMD_EXISTS=false
else
   SYSTEMD_EXISTS=true
fi

# - Running in a terminal?
# -
if [[ -t 1 ]] ; then
      terminal=true
else
   terminal=false
fi

#clear

echo ""
echo -e "\033[32m-----\033[m"
echo -e "Remove User from Group - \033[1mBoth, User and Group has to exist\033[m"
echo -e "\033[32m-----\033[m"


WEBSITE=
_OK=false
echo ""
echo "Cloud instance that you want to change?"
echo ""
declare -i i=0
for _site in ${website_arr[@]} ; do
   IFS=':' read -a _arr <<< ${_site}
   echo "   [$i] ${_arr[0]}"
   ((i++))
done
echo
echononl "   Eingabe: "
while ! $_OK ; do
read _IN
   if is_number "$_IN" && [[ -n ${website_arr[$_IN]} ]]; then
      IFS=':' read -a _arr <<< ${website_arr[$_IN]}
      conf_file=${_arr[1]}
      DEFAULT_WEBSITE="${_arr[0]}"
      _OK=true
   else
      echo ""  
      echo -e "\tFalsche Eingabe !"
      echo ""
      echononl "   Eingabe: "
   fi
done


# - Reset IFS
# -
IFS=$CUR_IFS


echo ""
echononl "   Include Configuration file.."
if [[ ! -f $conf_file ]]; then
   echo_failed
   fatal "Missing configuration file '$conf_file'"
else
   source $conf_file
   echo_ok
fi
echo ""



# =============
# --- Some checks
# =============

DEFAULT_SRC_BASE_DIR="/usr/local/src/nextcloud"
DEFAULT_HTTP_USER="www-data"
DEFAULT_HTTP_GROUP="www-data"
DEFAULT_PHP_ENGINE="FPM"


if [[ -z ${WEBSITE} ]] ; then
   fatal "No website given (parameter 'WEBSITE')"
fi

DEFAULT_WEB_BASE_DIR="/var/www/$WEBSITE"

[[ -n "$WEB_BASE_DIR" ]] || WEB_BASE_DIR=$DEFAULT_WEB_BASE_DIR

if [[ ! -d ${WEB_BASE_DIR} ]] ; then
   fatal "Web base directory not found (parameter 'WEB_BASE_DIR')"
fi

[[ -n "$PHP_ENGINE" ]] || PHP_ENGINE=$DEFAULT_PHP_ENGINE

if [[ "$DATABASE_TYPE" != "postgres" ]] && [[ "$DATABASE_TYPE" != "mysql" ]]; then
   fatal "Wrong or missing database type (parameter 'DATABASE_TYPE')"
fi

if [[ -z "$DATABASE_NAME" ]]; then
   fatal "Missing database name (parameter 'DATABASE_NAME')"
fi

if [[ "$DATABASE_TYPE" = "mysql" ]] && [[ -z "$MYSQL_CREDENTIALS" ]]; then
   fatal "No Database Credentials for MySQL given (parameter 'MYSQL_CREDENTIALS')"
fi

if [[ "$DATABASE_TYPE" = "postgres" ]]; then
   if [[ -z "$PSQL_USER" ]] || [[ -z "$PSQL_PASS" ]]; then
      fatal "No Database Credentials for PostgreSQL given (parameters: 'PSQL_USER' 'PSQL_PASS'"
   fi
fi

NGINX_IS_ENABLED=false
APACHE2_IS_ENABLED=false

# Check if NGINX webserver is ctive
#
if $(systemctl -q is-enabled nginx 2> /dev/null) ; then

   NGINX_IS_ENABLED=true

   # - Determin user of the webserver
   # -
   nginx_binary="$(which nginx)"
   if [[ -z "$nginx_binary" ]] ; then
      nginx_binary="$(ps -axu | grep -E "nginx:.*master" | grep -v grep | grep -o -E "\S+/nginx")"
      if [[ -z "$nginx_binary" ]] ; then
         if [[ -x "/usr/local/nginx/bin/nginx" ]]; then
            nginx_binary="/usr/local/nginx/bin/nginx"
         elif [[ -x "/usr/local/nginx/sbin/nginx" ]]; then
            nginx_binary="/usr/local/nginx/sbin/nginx"
         fi
      fi
   fi

   if [[ -x "$nginx_binary" ]] ; then
      _HTTP_USER="$($nginx_binary -T 2> /dev/null | grep -E "^\s*user\s+\S+;" | grep -o -E "\S+;$" | sed -e 's/;$//')"
   fi

   # - Is webserver running ?
   # -
   PID=$(ps aux | grep "$(realpath $nginx_binary)" | grep -e "^root" | grep -v grep | awk '{print$2}')
   if [[ "X${PID}X" = "XX" ]] ;then
      IS_HTTPD_RUNNING=false
   else
      IS_HTTPD_RUNNING=true
   fi


elif $(systemctl -q is-enabled apache2 2> /dev/null) ; then

   APACHE2_IS_ENABLED=true

   # - Determin user of the webserver
   # -
   httpd_binary="`which httpd`"
   if [ -z "$httpd_binary" ]; then
      httpd_binary="$(ps -axu | grep httpd |  grep -e "^root" | grep -v grep | awk '{print$11}')"
      if [ -z "$httpd_binary" ]; then
         if [ -x "/usr/local/apache2/bin/httpd" ]; then
            httpd_binary="/usr/local/apache2/bin/httpd"
         fi
      fi
   fi
   if [ -x "$httpd_binary" ];then

      # - Determin websever user
      # -
      _HTTP_USER="`$httpd_binary -t -D DUMP_RUN_CFG | grep -i -e "^User" | awk '{print$2}' | cut -d\"=\" -f2 | tr -d '"'`"
      _HTTP_GROUP="`$httpd_binary -t -D DUMP_RUN_CFG | grep -i -e "^Group" | awk '{print$2}' | cut -d\"=\" -f2 | tr -d '"'`"

      # - Is webserver running ?
      # -
      PID=$(ps aux | grep "$(realpath $httpd_binary)" | grep -e "^root" | grep -v grep | awk '{print$2}')
      if [[ "X${PID}X" = "XX" ]] ;then
         IS_HTTPD_RUNNING=false
      else
         IS_HTTPD_RUNNING=true
      fi
   fi

else
   error "Neither \033[1mapache2\033[m nor \033[1mnginx\033[m is enabled on this machine"

   echononl "continue anyway [yes/no]: "
   read OK
   OK="$(echo "$OK" | tr '[:upper:]' '[:lower:]')"
   while [[ "$OK" != "yes" ]] && [[  "$OK" != "no" ]] ; do
      echononl "Wrong entry! - repeat [yes/nno]: "
      read OK
   done
   [[ $OK = "yes" ]] || fatal "Interrupted by user"
fi

if [[ -n "$_HTTP_USER" ]] ; then
   if [[ -n "$HTTP_USER" ]] && [[ "$_HTTP_USER" != "$HTTP_USER" ]]; then
      warn "The script has determined \033[1;37m${_HTTP_USER}\033[m as Webservers user. This\n                     value differs from the value given in your configuration file, \n                     which is \033[1;37m${HTTP_USER}\033[m and takes precedence."
   else
      HTTP_USER=$_HTTP_USER
   fi
else
   [[ -n "$HTTP_USER" ]] || HTTP_USER=$DEFAULT_HTTP_USER
fi
if [[ -n "$_HTTP_GROUP" ]] ; then
   if [[ -n "$HTTP_GROUP" ]] && [[ "$_HTTP_GROUP" != "$HTTP_GROUP" ]]; then
      warn "The script has determined \033[1;37m${_HTTP_GROUP}\033[m as Webservers group. This\n                     value differs from the value given in your configuration file, \n                     which is \033[1;37m${HTTP_GROUP}\033[m and takes precedence."
   else
      HTTP_GROUP=$_HTTP_GROUP
   fi
else
   [[ -n "$HTTP_GROUP" ]] || HTTP_GROUP=$DEFAULT_HTTP_GROUP
fi

# Check PHP Version
#
source ${snippet_dir}/get-php-major-version

CURRENT_INSTALL_DIR=`realpath ${WEB_BASE_DIR}/nextcloud`
CURRENT_DATA_DIR=`realpath ${WEB_BASE_DIR}/data`
CURRENT_VERSION=`basename $CURRENT_INSTALL_DIR | cut -d"-" -f2`


echo ""
echo ""
echo -e "\033[32m-----\033[m"
echo ""
echo -e "   From which \033[1mGroup\033[m do you want to remove a User?"
echo ""
GROUP=
echononl "   Enter Group: "
read GROUP
while [[ -z "$(trim $GROUP)" ]] ; do
   echo -e "\n\t\033[33m\033[1mA group name must be entered.\033[m\n"
   echononl "   Enter Group: "
   read GROUP
done


echo ""
echo ""
echo -e "\033[32m-----\033[m"
echo ""
echo -e "   Which \033[1mUser\033[m would you like to remove from group \033[1m$GROUP\033[m?"
echo ""
USER=
echononl "   Enter User: "
read USER
while [[ -z "$(trim $USER)" ]] ; do
   echo -e "\n\t\033[33m\033[1mA user must be entered.\033[m\n"
   echononl "   Enter User: "
   read USER
done



echo ""
echo -e "\033[1;32mStarting Script for \033[1;37m${WEBSITE}\033[m"
echo ""
echo -e   "   Cloud instance to be changed.........: $WEBSITE"
echo ""
echo -e   "   Current version of nextcloud.........: $CURRENT_VERSION"
echo ""
echo ""
echo -e   "   Gruppe...............................: $GROUP"
echo -e   "   User.................................: $USER"
echo ""
echo ""
echo -e   "   Web base directory...................: $WEB_BASE_DIR"
echo ""
echo -e   "   Webserver user.......................: $HTTP_USER"
echo -e   "   Webserver group......................: $HTTP_GROUP"
echo ""
echo -e   "   PHP version..........................: $PHP_VERSION"
echo -e   "   PHP Engine...........................: $PHP_ENGINE"
echo ""
echo -e   "   Databse name.........................: $DATABASE_NAME"
echo -e   "   Database type........................: $DATABASE_TYPE"
echo ""
if [[ "$DATABASE_TYPE" = "mysql" ]] ; then
   echo -e   "   Mysql Credentials....................: $MYSQL_CREDENTIALS"
fi
echo ""

echo ""
echo -n "   Type upper case 'YES' to continue executing with this parameters: "
read OK
if [[ "$OK" = "YES" ]] ; then
   echo ""
   echo ""
   echo -e "\033[1;32mGoing to remove user \033[1;37m$USER\033[1;32m from group \033[1;37m$GROUP033[1;32m on \033[1;37m$WEBSITE \033[m"
   echo ""
else
   fatal "Abort by user request - Answer as not 'YES'"
fi


echo ""


# -----
# - Main part of the script
# -----

echo ""
echo ""
echo -e "\033[37m\033[1mMain part of the script\033[m"
echo ""


# - Create missing indices
# -
echononl "   Remove user \033[37m\033[1m$USER\033[m from group \033[37m\033[1m$GROUP\033[m .."

su -c "/usr/local/php/bin/php ${WEB_BASE_DIR}/htdocs/occ group:removeuser '$GROUP' '$USER'" -s /bin/bash $HTTP_USER > $log_file 2>&1
if [[ $? -eq 0 ]]; then
   echo_ok
else
   echo_failed
   blank_line
   echo ""
   echo -e "\033[37m\033[1mcommandline was:\033[m"
   echo -e "su -c \"/usr/local/php/bin/php ${WEB_BASE_DIR}/htdocs/occ group:removeuser '$GROUP' '$USER'\" -s /bin/bash $HTTP_USER"
   blank_line
   warn "Despite errors, it is possible that the user \033[37m\033[1m$USER\033[m was removed from the group \033[37m\033[1m$GROUP\033[m"
   fatal "$(cat $log_file)"
fi

blank_line
clean_up 0
